#!/usr/bin/env python2

# PYTHON_ARGCOMPLETE_OK

# http://mikekneller.com/kb/python/libxml2python/part1

import sys, re, os,  argparse, argcomplete

from io import StringIO

from GoldenChild.xpath import *
from Perdy.parser import *
from Perdy.pretty import *
from Perdy.eddo import *

def argue():
	parser = argparse.ArgumentParser()
	
	parser.add_argument('-?',			 action='help',	   help='show this help')
	parser.add_argument('-v','--verbose', action='store_true', help='show detailed output')
	parser.add_argument('-b','--bare',	action='store_true', help='show bare output with root element')
	parser.add_argument('-u','--urls',	action='store_true', help='show namespace urls')
	parser.add_argument('-c','--colour',  action='store_true', help='show colour output')
	parser.add_argument('-t','--text',	action='store_true', help='print result as text')
	parser.add_argument('-s','--single',  action='store_true', help='display result as a single value')
	parser.add_argument('-d','--delete',  action='store_true', help='delete nodes by xpath')
	parser.add_argument('-i','--inline',  action='store_true', help='when deleting do it to the original file')
	parser.add_argument('-p','--pretty',  action='store_true', help='display horizontal bar between files')
	parser.add_argument('-z','--horizon', action='store_true', help='display horizontal bar between files')
	parser.add_argument('-f','--fname',   action='store_true', help='show file name')
	parser.add_argument('-o','--output',  action='store',	  help='output to file')
	parser.add_argument('-e','--element', action='store',	  help='use this element as the document root', default='results')
	parser.add_argument('-n','--ns',	  action='store',	  help='added to context ', nargs='*', metavar='prefix=\"url\"')
	parser.add_argument('-x','--xpath',   action='store',	  help='xpath to apply to the file')
	parser.add_argument('file',		   action='store',	  help='file to parse', nargs='*')

	argcomplete.autocomplete(parser)
	args = parser.parse_args()

	if args.verbose:
		sys.stderr.write('args : ')
		prettyPrint(vars(args), colour=True, output=sys.stderr)

	return args
		
def element(xml,rdoc,rctx,nsp):
	(doc,ctx,_nsp) = getContextFromString(xml, urls=args.urls)
	element = doc.getRootElement().copyNode(True)
	for ns in nsp.keys():
		ctx.xpathRegisterNs(ns,nsp[ns])
		#element.setProp('xmlns:%s'%ns,'%s'%nsp[ns])
		rdoc.getRootElement().setProp('xmlns:%s'%ns,'%s'%nsp[ns])
	rdoc.getRootElement().addChild(element)
	return

def process(xml,output=sys.stdout,rdoc=None,rctx=None):
	if True: #try
		(doc,ctx,nsp)=getContextFromStringWithNS(xml, args.ns, urls=args.urls)
		
		if args.verbose:
			sys.stderr.write('nsp : ')
			#print(nsp)
			prettyPrint(nsp,colour=True,output=sys.stderr)

		res = ctx.xpathEval(args.xpath)
		if args.delete:
			for r in res:
				r.unlinkNode()
				r.freeNode()
			output.write('%s\n'%doc)
		elif args.single:
			if args.pretty:
				sp = StringIO.StringIO('%s'%res)
				doParse(sp,output,colour=args.colour)
				sp.close()
			else:
				output.write('%s\n'%res)
		else:
			if len(res) == 0:
				None
				#output.write('\n')
			else:
				for r in res:
					if args.text:
						output.write('%s\n'%r.content)
					elif not args.bare and rdoc and rctx:
						element('%s'%r,rdoc,rctx,nsp)
					else:
						if args.pretty:
							sp = StringIO.StringIO('%s'%r)
							doParse(sp,output,colour=args.colour)
							sp.close()
						else:
							output.write('%s\n'%r)

	if False: #except:
		sys.stderr.write('<!-- exception when parsing -->\n')
		if args.verbose:
			sys.stderr.write('exc_info : ')
			prettyPrint(sys.exc_info(), output=sys.stderr)

	return

def main():
	global args

	args = argue()
	if args.horizon:
		horizon = buildHorizon()
	else:
		horizon = None

	if args.output:
		output = open(args.output,'w')
		sys.stderr.write('%s\n'%args.output)
	else:
		output=sys.stdout

	(rdoc,rctx) = (None,None)
	if not args.bare:
		(rdoc,rctx,nsp) = getContextFromString('<%s/>'%args.element, urls=args.urls)

	if args.file and len(args.file) > 0:
		for file in args.file:
			if horizon:
				sys.stderr.write('%s\n'%horizon)
			if args.fname:
				if args.text or args.single:
					sys.stderr.write('%s: '%file)
				else:
					sys.stderr.write('%s\n'%file)

			fp = open(file)
			xml=fp.read()
			fp.close()

			if args.inline: output = open(file,'w')
			process(xml,output,rdoc,rctx)
			if args.inline:
				print file
				output.close()
			
	else:
		xml = sys.stdin.read()
		process(xml,output,rdoc,rctx)

	if not args.delete and not args.bare and not args.text and not args.single:
		if args.pretty:
			sp = StringIO.StringIO('%s'%rdoc)
			doParse(sp,output,colour=args.colour)
			sp.close()
		else:
			output.write('%s'%rdoc)
		
	if args.output:
		output.close()

	return

if __name__ == '__main__' : main()



#!/usr/bin/env python2

# PYTHON_ARGCOMPLETE_OK

import sys, re, os, libxml2, argparse, argcomplete

from Perdy.Colours import Colours
from Perdy.eddo import *
from Perdy.pretty import *
from GoldenChild.xpath import *

parser = argparse.ArgumentParser()

parser.add_argument('-?',             action='help',       help='show this help')
parser.add_argument('-v','--verbose', action='store_true', help='show detailed output')
parser.add_argument('-s','--show',    action='store_true', help='show changes')
parser.add_argument('-c','--cdata',   action='store_true', help='text as cdata')
parser.add_argument('-t','--text',    action='store',      help='set value as text value')
parser.add_argument('-T','--tfile',   action='store',      help='set value as text value taken from tfile')
parser.add_argument('-a','--attr',    action='store',      help='set value as attribute value')
parser.add_argument('-x','--xpath',   action='store',      help='xpath to apply to the file')
parser.add_argument('-n','--ns',      action='store',      help='added to context ', nargs='*', metavar='prefix=\"url\"')
parser.add_argument('file',           action='store',      help='file to parse', nargs='*')

argcomplete.autocomplete(parser)
args = parser.parse_args()

if args.verbose:
    prettyPrint(('args',vars(args)), colour=True, output=sys.stderr)

def main():

    pn = re.compile('^([^=]*)=["\']([^\'"]*)["\']$')
    ns = {}
    if args.ns:
        for nsp in args.ns:
            m = pn.match(nsp)
            if m:
                ns[m.group(1)]=m.group(2)
        if args.verbose:
            prettyPrint(('ns',ns), colour=True, output=sys.stderr)
            
    if args.attr and args.xpath:
        pa = re.compile('^(.*)/@([^@]*)$')
        m = pa.match(args.xpath)
        if m:
            xpath=m.group(1)
            attr = m.group(2)
            if args.verbose:
                sys.stderr.write('attr name=%s\n'%attr)
        else:
            sys.stderr.write('can\'t match attr from xpath=%s'%args.xpath)
    else:
        xpath = args.xpath

    for f in args.file:
        if args.show:
            sys.stderr.write('%s\n'%f)
        b='%s.bak'%f
        try:
            os.remove(b)
        except:
            None
        os.rename(f,b)
        
        (doc,ctx,nsp)=getContextFromFile(b)
        for p in ns.keys():
            ctx.xpathRegisterNs(p,ns[p])

        if args.tfile:
            with open(args.tfile) as tf:
                text = ''.join(tf.readlines())
                tf.close()
        elif args.text:
            text = args.text
        else:
            text = None
            
        res = ctx.xpathEval(xpath)
        for r in res:
            if text:
                if args.show:
                    sys.stderr.write('%s- %s%s\n'%(colours['Red'],r.content,colours['Off']))
                    sys.stderr.write('%s+ %s%s\n'%(colours['Green'],text,colours['Off']))
                if args.cdata:
                    r.setContent(None)
                    cdata = doc.newCDataBlock(text,len(text))
                    r.addChild(cdata)
                else:
                    r.setContent(text)
            if args.attr:
                if args.show:
                    sys.stderr.write('%s- %s%s\n'%(colours['Red'],r.prop(attr),colours['Off']))
                    sys.stderr.write('%s+ %s%s\n'%(colours['Green'],args.attr,colours['Off']))
                r.setProp(attr,args.attr)
                
        output = open(f,'w')
        output.write('%s'%doc)
        output.close()
    return

if __name__ == '__main__' : main()



#!/usr/bin/env python2

import sys, re, os, libxml2
from collections import namedtuple

XML = namedtuple('XML', ['doc','ctx','nsp'])

def getContextFromStringWithNS(xml, argsNS=None, urls=None):

	def handler(ectx, error):
		None #sys.stderr.write('error=%s'%error)

	libxml2.registerErrorHandler(handler,'')

	doc = libxml2.parseDoc(xml)
	ctx = doc.xpathNewContext()
	nsp = {}

	for ns in [
		doc.getRootElement().ns(),
		doc.getRootElement().nsDefs(),
	]:
		if ns:
			nsp[ns.name] = ns.content
			if urls:
				sys.stderr.write('xmlns:%s=%s\n'%(ns.name, ns.content))

	if argsNS:
		pn = re.compile('^([^=]*)=["\']([^\'"]*)["\']$')
		for ns in argsNS:
			mn = pn.match(ns)
			if mn:
				nsp[mn.group(1)] = mn.group(2)

	for prefix, namespace in nsp.iteritems():
		ctx.xpathRegisterNs(prefix,namespace)

	return (doc,ctx,nsp)

def getContextFromString(xml, argsNS=None, urls=None):
	(doc,ctx,nsp) = getContextFromStringWithNS(xml, argsNS, urls)
	return (doc,ctx,nsp)

def getContextFromFile(fn, argsNS=None, urls=None):
	fp = open(fn)
	xml = fp.read()
	fp.close()
	return getContextFromString(xml, argsNS, urls)

def getContextFromStream(input, argsNS=None, urls=None):
	xml = input.read()
	return getContextFromString(xml, argsNS, urls)

def getElements(context,xpath,parent=None):
	if parent:
		context.setContextNode(parent)
	return context.xpathEval(xpath)

def getElement(context,xpath,parent=None):
	elements = getElements(context,xpath,parent)
	if len(elements) > 0:
		return elements[0]
	return None

def getElementText(context,xpath,parent=None):
	element = getElement(context,xpath,parent)
	if element:
		return element.content
	return None

def setElementText(context,xpath,value,parent=None):
	element = getElement(context,xpath,parent)
	if element:
		element.setContent(value)
	return

def insertElement(document, name, before, parent=None, ns=None):
	if not parent:
		parent = document.getRootElement()
	#print(parent.ns(), type(parent.ns()))
	element = document.newDocNode(ns or parent.ns(), name, None)
	before.addPrevSibling(element)
	return element

def appendElement(document, name, after, parent=None, ns=None):
	if not parent:
		parent = document.getRootElement()
	#print(parent.ns(), type(parent.ns()))
	element = document.newDocNode(ns or parent.ns(), name, None)
	after.addNextSibling(element)
	return element

def addElement(document, name, parent=None, ns=None):
	if not parent:
		parent = document.getRootElement()
	#print(parent.ns(), type(parent.ns()))
	element = document.newDocNode(ns or parent.ns(), name, None)
	parent.addChild(element)
	return element

def addElementText(document, name, value, parent=None, ns=None):
	if not parent:
		parent = document.getRootElement()
	element = document.newDocNode(ns or parent.ns(),name,value)
	parent.addChild(element)
	return element

def addElementCDATA(document,name,value,parent=None):
	element = addElement(document,name,parent=parent)
	cdata = document.newCDataBlock(value,len(value))
	element.addChild(cdata)
	return element

def hasAttribute(element,attname):
	return element.hasProp(attname)

def getAttribute(element,attname):
	return element.prop(attname)

def setAttribute(element,attname,value):
	try:
		element.setProp(attname,value)
	except:
		element.newProp(attname,value)
	return

def delAttribute(element,attname):
	for property in element.properties:
		#sys.stderr.write('%s\n'%property)
		if property.type == 'attribute' and property.name == attname:
			property.unlinkNode()
			property.freeNode()
			break
	return
